@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/functions.h,v 1.38 2007/08/22 10:10:23 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/functions.h,v 1.46 2009/10/16 09:10:40 tom Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -35,6 +35,7 @@ extern int     tls_server_start(uschar *, uschar *, uschar *, uschar *);
 extern BOOL    tls_smtp_buffered(void);
 extern int     tls_ungetc(int);
 extern int     tls_write(const uschar *, size_t);
+extern void    tls_version_report(FILE *);
 #endif
 
 
@@ -63,6 +64,11 @@ extern pid_t   child_open_uid(uschar **, uschar **, int, uid_t *, gid_t *,
                  int *, int *, uschar *, BOOL);
 
 extern void    daemon_go(void);
+
+#ifdef EXPERIMENTAL_DCC
+extern int     dcc_process(uschar **);
+#endif
+
 extern void    debug_print_argv(uschar **);
 extern void    debug_print_ids(uschar *);
 extern void    debug_print_string(uschar *);
@@ -78,8 +84,8 @@ extern void    deliver_succeeded(address_item *);
 extern int     demime(uschar **);
 #endif
 extern BOOL    directory_make(uschar *, uschar *, int, BOOL);
-#ifdef EXPERIMENTAL_DOMAINKEYS
-extern BOOL    dk_transport_write_message(address_item *, int, int,
+#ifndef DISABLE_DKIM
+extern BOOL    dkim_transport_write_message(address_item *, int, int,
                    int, uschar *, uschar *, uschar *, uschar *, rewrite_rule *,
                    int, uschar *, uschar *, uschar *, uschar *, uschar *, uschar *);
 #endif
@@ -183,6 +189,8 @@ extern uschar *parse_find_at(uschar *);
 extern uschar *parse_fix_phrase(uschar *, int, uschar *, int);
 extern uschar *parse_message_id(uschar *, uschar **, uschar **);
 extern uschar *parse_quote_2047(uschar *, int, uschar *, uschar *, int, BOOL);
+extern uschar *parse_date_time(uschar *str, time_t *t);
+extern int     pseudo_random_number(int);
 
 extern BOOL    queue_action(uschar *, int, uschar **, int, int);
 extern void    queue_check_only(void);
@@ -195,8 +203,8 @@ extern int     random_number(int);
 extern int     recv_line(int, uschar *, int);
 #endif
 extern int     rda_interpret(redirect_block *, int, uschar *, uschar *,
-                 uschar *, uschar *, ugid_block *, address_item **, uschar **,
-                 error_block **, int *, uschar *);
+                 uschar *, uschar *, uschar *, ugid_block *, address_item **,
+                 uschar **, error_block **, int *, uschar *);
 extern int     rda_is_filter(const uschar *);
 extern BOOL    readconf_depends(driver_instance *, uschar *);
 extern void    readconf_driver_init(uschar *, driver_instance **,
@@ -260,7 +268,7 @@ extern void    sha1_end(sha1 *, const uschar *, int, uschar *);
 extern void    sha1_mid(sha1 *, const uschar *);
 extern void    sha1_start(sha1 *);
 extern int     sieve_interpret(uschar *, int, uschar *, uschar *, uschar *,
-                 address_item **, uschar **);
+                 uschar *, address_item **, uschar **);
 extern void    sigalrm_handler(int);
 extern BOOL    smtp_buffered(void);
 extern void    smtp_closedown(uschar *);
