@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/mytypes.h,v 1.4 2007/01/08 10:50:18 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/mytypes.h,v 1.6 2009/10/19 12:29:31 nm4 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -15,19 +15,26 @@ local_scan.h includes it and exim.h includes them both (to get this earlier). */
 #ifndef MYTYPES_H
 #define MYTYPES_H
 
-
+#ifndef FALSE
 #define FALSE         0
+#endif
+
+#ifndef TRUE
 #define TRUE          1
+#endif
+
+#ifndef TRUE_UNSET
 #define TRUE_UNSET    2
+#endif
 
 
 /* If gcc is being used to compile Exim, we can use its facility for checking
 the arguments of printf-like functions. This is done by a macro. */
 
 #ifdef __GNUC__
-#define PRINTF_FUNCTION  __attribute__((format(printf,1,2)))
+#define PRINTF_FUNCTION(A,B)  __attribute__((format(printf,A,B)))
 #else
-#define PRINTF_FUNCTION
+#define PRINTF_FUNCTION(A,B)
 #endif
 
 
